@@ -1,9 +1,11 @@
 .onLoad <- function(...) {
   vctrs::s3_register("parsnip::multi_predict", "_tabnet_fit")
   vctrs::s3_register("vip::vi_model", "tabnet_fit")
+  vctrs::s3_register("vip::vi_model", "tabnet_pretrain")
   vctrs::s3_register("ggplot2::autoplot", "tabnet_fit")
   vctrs::s3_register("ggplot2::autoplot", "tabnet_pretrain")
   vctrs::s3_register("ggplot2::autoplot", "tabnet_explain")
+  vctrs::s3_register("tune::min_grid", "tabnet")
 }
 
 
